STATUS = {
    200:"OK",

    400:"Bad request",
    451:"Message must be a dictionary",
    452:"Missing 'action' Field",
    453:"Name not allowed",
    454:"'name' Already Subscribed",
    455:"Missing 'to' Field",
    456:"Client Not Subscribed",
    457:"Missing 'msg' Field",
    458:"'subscribe' is the only First Action Allowed",
    459:"Unknown 'action' value"
    }